{-# LANGUAGE DeriveAnyClass #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DerivingStrategies #-}
{-# LANGUAGE OverloadedStrings #-}

module Agent.Server.Config
  ( configPath,
    loadConfig,
    loadConfigOrDefault,
    saveConfig,
    Config (..),
    OllamaConfig (..),
    NoConfigFoundException,
    configFile,
  )
where

import Control.Exception (Exception (..))
import Control.Monad.Extra (whenMaybeM)
import Data.Default (Default (..))
import Data.Text (Text (..))
import Data.Yaml (FromJSON, ParseException (..), ToJSON, decodeFileEither, decodeFileThrow, encodeFile)
import GHC.Generics (Generic)
import System.Directory (XdgDirectory (..), createDirectoryIfMissing, doesFileExist, getXdgDirectory)
import System.FilePath ((</>))

data Config = Config
  { startOllama :: Bool,
    ollama :: OllamaConfig
  }
  deriving stock (Show, Eq, Generic)
  deriving anyclass (FromJSON, ToJSON)

instance Default Config where
  def =
    Config
      { startOllama = False,
        ollama = def
      }

data OllamaConfig = OllamaConfig
  { host :: Text,
    port :: Int,
    cmdExtra :: Text
  }
  deriving stock (Show, Eq, Generic)
  deriving anyclass (FromJSON, ToJSON)

instance Default OllamaConfig where
  def =
    OllamaConfig
      { host = "localhost",
        port = 11434,
        cmdExtra = ""
      }

data NoConfigFoundException = NotFound FilePath
  deriving (Show)

instance Exception NoConfigFoundException

configPath :: FilePath
configPath = "hAgent"

configName :: FilePath
configName = "config.yaml"

configFile :: IO (Maybe FilePath)
configFile = do
  filepath <- (getXdgDirectory XdgConfig (configPath </> configName))
  whenMaybeM (doesFileExist filepath) (return filepath)

loadConfig :: IO (Either ParseException Config)
loadConfig = do
  filepath <- (getXdgDirectory XdgConfig (configPath </> configName))
  confFile <- configFile
  case confFile of
    Just f -> decodeFileEither f
    Nothing -> return $ Left $ OtherParseException $ toException $ NotFound $ filepath

loadConfigOrDefault :: IO (Either ParseException Config)
loadConfigOrDefault = do
  confFile <- configFile
  case confFile of
    Just f -> decodeFileEither f
    Nothing -> return $ Right $ def

saveConfig :: Config -> IO ()
saveConfig conf = do
  getXdgDirectory XdgConfig configPath >>= createDirectoryIfMissing True
  confPath <- (getXdgDirectory XdgConfig (configPath </> configName))
  encodeFile confPath conf
